@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015 Adrien Vergé
+ *  Copyright (c) 2015 Adrien Vergé
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -366,7 +366,7 @@ int load_config(struct vpn_config *cfg, const char *filename)
 			int use_resolvconf = strtob(val);
 
 			if (use_resolvconf < 0) {
-				log_warn("Bad use-resolvconf value in configuration file: \"%s\".\n",
+				log_warn("Bad value for use-resolvconf in configuration file: \"%s\".\n",
 				         val);
 				continue;
 			}
