@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015 Adrien Vergé
+ *  Copyright (c) 2015 Adrien Vergé
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -186,6 +186,27 @@ PPPD_USAGE \
 "      trusted-cert = othercertificatedigest6631bf...\n" \
 "  For a full-featured configuration see man openfortivpn(1).\n"
 
+/**
+ * Returns the given "input" prefixed with "prefix" in a dynamically
+ * allocated string. This behaves exactly like "strdup" if "input" already
+ * starts with "prefix".
+ */
+static char *strdup_with_prefix(const char *input, const char *prefix)
+{
+	size_t prefix_len = strlen(prefix);
+	char *output;
+
+	if (strncmp(prefix, input, prefix_len) == 0)
+		return strdup(input);
+
+	output = malloc(prefix_len + strlen(input) + 1);
+	if (output) {
+		strcpy(output, prefix);
+		strcpy(output + prefix_len, input);
+	}
+	return output;
+}
+
 int main(int argc, char **argv)
 {
 	int ret = EXIT_FAILURE;
@@ -317,7 +338,7 @@ int main(int argc, char **argv)
 			           "version") == 0) {
 				printf(VERSION "\n");
 				if (strcmp(&REVISION[1], VERSION))
-					log_debug("revision " REVISION "\n");
+					log_debug("Revision " REVISION "\n");
 				ret = EXIT_SUCCESS;
 				goto exit;
 			}
@@ -336,26 +357,31 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pppd-log") == 0) {
+				free(cli_cfg.pppd_log);
 				cli_cfg.pppd_log = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pppd-plugin") == 0) {
+				free(cli_cfg.pppd_plugin);
 				cli_cfg.pppd_plugin = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pppd-ifname") == 0) {
+				free(cli_cfg.pppd_ifname);
 				cli_cfg.pppd_ifname = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pppd-ipparam") == 0) {
+				free(cli_cfg.pppd_ipparam);
 				cli_cfg.pppd_ipparam = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pppd-call") == 0) {
+				free(cli_cfg.pppd_call);
 				cli_cfg.pppd_call = strdup(optarg);
 				break;
 			}
@@ -363,6 +389,7 @@ int main(int argc, char **argv)
 			if (cli_cfg.pppd_plugin == NULL &&
 			    strcmp(long_options[option_index].name,
 			           "plugin") == 0) {
+				free(cli_cfg.pppd_plugin);
 				cli_cfg.pppd_plugin = strdup(optarg);
 				break;
 			}
@@ -370,6 +397,7 @@ int main(int argc, char **argv)
 #if HAVE_USR_SBIN_PPP
 			if (strcmp(long_options[option_index].name,
 			           "ppp-system") == 0) {
+				free(cli_cfg.ppp_system);
 				cfg.ppp_system = strdup(optarg);
 				break;
 			}
@@ -390,16 +418,19 @@ int main(int argc, char **argv)
 #endif
 			if (strcmp(long_options[option_index].name,
 			           "ca-file") == 0) {
+				free(cli_cfg.ca_file);
 				cli_cfg.ca_file = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "user-cert") == 0) {
+				free(cli_cfg.user_cert);
 				cli_cfg.user_cert = strdup(optarg);
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
 			           "user-key") == 0) {
+				free(cli_cfg.user_key);
 				cli_cfg.user_key = strdup(optarg);
 				break;
 			}
@@ -413,6 +444,7 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "pinentry") == 0) {
+				free(cli_cfg.pinentry);
 				cli_cfg.pinentry = strdup(optarg);
 				break;
 			}
@@ -430,6 +462,7 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "cipher-list") == 0) {
+				free(cli_cfg.cipher_list);
 				cli_cfg.cipher_list = strdup(optarg);
 				break;
 			}
@@ -447,6 +480,7 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "otp-prompt") == 0) {
+				free(cli_cfg.otp_prompt);
 				cli_cfg.otp_prompt = strdup(optarg);
 				break;
 			}
@@ -517,7 +551,8 @@ int main(int argc, char **argv)
 			}
 			if (strcmp(long_options[option_index].name,
 			           "cookie") == 0) {
-				cli_cfg.cookie = strdup(optarg);
+				free(cli_cfg.cookie);
+				cli_cfg.cookie = strdup_with_prefix(optarg, "SVPNCOOKIE=");
 				break;
 			}
 			if (strcmp(long_options[option_index].name,
@@ -525,11 +560,12 @@ int main(int argc, char **argv)
 				char *cookie = read_from_stdin(COOKIE_SIZE);
 
 				if (cookie == NULL) {
-					log_warn("Could not read the cookie from stdin");
+					log_error("Could not read the cookie from stdin\n");
 					break;
 				}
 				free(cli_cfg.cookie);
-				cli_cfg.cookie = cookie;
+				cli_cfg.cookie = strdup_with_prefix(cookie, "SVPNCOOKIE=");
+				free(cookie);
 				break;
 			}
 			goto user_error;
