@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015 Adrien Vergé
+ *  Copyright (c) 2015 Adrien Vergé
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,9 +30,6 @@
 #include "http.h"
 #include "log.h"
 #include "userinput.h"
-#ifndef HAVE_X509_CHECK_HOST
-#include "openssl_hostname_validation.h"
-#endif
 
 #include <openssl/err.h>
 #ifndef OPENSSL_NO_ENGINE
@@ -383,11 +380,12 @@ static const char * const ppp_message[] = {
 	"The PPP negotiation failed because serial loopback was detected.",
 	"The init script failed (returned a non-zero exit status).",
 	"We failed to authenticate ourselves to the peer."
-#else // sysexits(3) - https://www.freebsd.org/cgi/man.cgi?query=sysexits
+#endif
+#if HAVE_USR_SBIN_PPP // sysexits(3) - https://www.freebsd.org/cgi/man.cgi?query=sysexits
 	// EX_NORMAL = EX_OK (0)
 	"Successful exit.",
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, // 1-9
-	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,	// 10-19
+	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, // 10-19
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, // 20-29
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, // 30-39
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, // 40-49
@@ -751,7 +749,7 @@ static int tcp_connect(struct tunnel *tunnel)
 	log_debug("server_port: %u\n", ntohs(server.sin_port));
 	server.sin_family = AF_INET;
 	memset(&(server.sin_zero), '\0', 8);
-	log_debug("gateway_addr: %s\n", inet_ntoa(tunnel->config->gateway_ip));
+	log_debug("gateway_ip: %s\n", inet_ntoa(tunnel->config->gateway_ip));
 	log_debug("gateway_port: %u\n", tunnel->config->gateway_port);
 
 	ret = connect(handle, (struct sockaddr *) &server, sizeof(server));
@@ -855,7 +853,6 @@ err_socket:
 static int ssl_verify_cert(struct tunnel *tunnel)
 {
 	int ret = -1;
-	int cert_valid = 0;
 	unsigned char digest[SHA256LEN];
 	unsigned int len;
 	struct x509_digest *elem;
@@ -874,25 +871,13 @@ static int ssl_verify_cert(struct tunnel *tunnel)
 		return 1;
 	}
 
-	subj = X509_get_subject_name(cert);
-
-#ifdef HAVE_X509_CHECK_HOST
-	// Use OpenSSL native host validation if v >= 1.0.2.
-	// compare against gateway_host and correctly check return value
-	// to fix prior incorrect use of X509_check_host
-	if (X509_check_host(cert, tunnel->config->gateway_host,
-	                    0, 0, NULL) == 1)
-		cert_valid = 1;
-#else
-	// Use validate_hostname form iSECPartners if native validation not available
-	// in order to avoid TLS Certificate CommonName NULL Byte Vulnerability
-	if (validate_hostname(tunnel->config->gateway_host, cert) == MatchFound)
-		cert_valid = 1;
-#endif
-
-	// Try to validate certificate using local PKI
-	if (cert_valid
-	    && SSL_get_verify_result(tunnel->ssl_handle) == X509_V_OK) {
+	// Validate certificate:
+	// 1. Validate using local PKI
+	// 2. Compare against gateway_host and correctly check return value
+	//    to fix prior incorrect use of X509_check_host
+	if (SSL_get_verify_result(tunnel->ssl_handle) == X509_V_OK
+	    && X509_check_host(cert, tunnel->config->gateway_host,
+	                       0, 0, NULL) == 1) {
 		log_debug("Gateway certificate validation succeeded.\n");
 		ret = 0;
 		goto free_cert;
@@ -920,6 +905,8 @@ static int ssl_verify_cert(struct tunnel *tunnel)
 		goto free_cert;
 	}
 
+	subj = X509_get_subject_name(cert);
+
 	log_error("Gateway certificate validation failed, and the certificate digest is not in the local whitelist. If you trust it, rerun with:\n");
 	log_error("    --trusted-cert %s\n", digest_str);
 	log_error("or add this line to your configuration file:\n");
@@ -987,7 +974,7 @@ static int pem_passphrase_cb(char *buf, int size, int rwflag, void *u)
 				              PEM_PASSPHRASE_SIZE + 1);
 				cfg->pem_passphrase_set =  1;
 			} else {
-				log_error("Buffer too small for PEM pass phrase: %d.",
+				log_error("Buffer too small for PEM pass phrase: %d.\n",
 				          size);
 			}
 		}
@@ -998,7 +985,7 @@ static int pem_passphrase_cb(char *buf, int size, int rwflag, void *u)
 			return strlen(buf);
 		}
 	} else {
-		log_error("We refuse to write PEM pass phrases!");
+		log_error("We refuse to write PEM pass phrases!\n");
 	}
 
 	return -1;
@@ -1115,8 +1102,8 @@ int ssl_connect(struct tunnel *tunnel)
 	}
 #endif
 
-	/* Use engine for PIV if user-cert config starts with pkcs11 URI: */
 #ifndef OPENSSL_NO_ENGINE
+	/* Use PKCS11 engine for PIV if user-cert config starts with pkcs11 URI: */
 	if (tunnel->config->use_engine > 0) {
 		ENGINE *e;
 
@@ -1175,10 +1162,8 @@ int ssl_connect(struct tunnel *tunnel)
 			          ERR_error_string(ERR_peek_last_error(), NULL));
 			goto err_ssl_context;
 		}
-
-	} else {        /* end PKCS11-engine */
+	} else { /* end PKCS11 engine */
 #endif
-
 		if (tunnel->config->user_cert) {
 			if (!SSL_CTX_use_certificate_chain_file(
 			            tunnel->ssl_context, tunnel->config->user_cert)) {
@@ -1347,7 +1332,7 @@ int run_tunnel(struct vpn_config *config)
 	log_debug("Starting IO through the tunnel\n");
 	io_loop(&tunnel);
 
-	log_debug("disconnecting\n");
+	log_debug("Disconnecting\n");
 	if (tunnel.state == STATE_UP)
 		if (tunnel.on_ppp_if_down != NULL)
 			tunnel.on_ppp_if_down(&tunnel);
